package cn.iocoder.yudao.module.reward.service.redeem;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.mybatis.core.service.AbstractService;
import cn.iocoder.yudao.framework.security.core.util.SecurityFrameworkUtils;
import cn.iocoder.yudao.module.reward.dal.dataobject.redeem.RewardRedeemDO;
import cn.iocoder.yudao.module.reward.dal.mysql.redeem.RewardRedeemMapper;
import cn.iocoder.yudao.module.reward.enums.ErrorCodeConstants;
import cn.iocoder.yudao.module.reward.enums.RewardRedeemStatusEnum;
import cn.iocoder.yudao.module.reward.service.reward.RewardService;
import cn.iocoder.yudao.module.reward.vo.reward.RewardRedeemPageReqVO;
import cn.iocoder.yudao.module.reward.vo.reward.RewardRedeemPageRespVO;
import cn.iocoder.yudao.module.reward.vo.reward.RewardRedeemUpdateVO;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.Date;
import java.util.List;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;

/**
 * 礼品兑换 Service
 *
 * @author 系统管理员
 */
@Service
@Slf4j
public class RewardRedeemServiceImpl extends AbstractService<RewardRedeemMapper, RewardRedeemDO> implements RewardRedeemService {
    @Resource
    private RewardRedeemMapper rewardRedeemMapper;
    @Resource
    private RewardService rewardService;

    @Override
    public PageResult<RewardRedeemPageRespVO> pageInfo(RewardRedeemPageReqVO reqVO) {
        int start = (reqVO.getPageNo() - 1) * reqVO.getPageSize();
        int size = reqVO.getPageSize();
        List<RewardRedeemPageRespVO > result = rewardRedeemMapper.pageInfo(start, size, reqVO);
        int total = rewardRedeemMapper.pageCount(reqVO);
        return new PageResult<>(result, total, reqVO.getPageSize(), reqVO.getPageNo(), (total + reqVO.getPageSize() - 1) / reqVO.getPageSize());
    }

    @Override
    public RewardRedeemPageRespVO detail(Long id) {
        if (id == null) {
            return null;
        }
        return rewardRedeemMapper.detail(id);
    }

    @Override
    public Boolean updateRecord(RewardRedeemUpdateVO request) {
        validateParam(request);
        LambdaUpdateWrapper<RewardRedeemDO> wrapper = Wrappers.lambdaUpdate();
        wrapper.eq(RewardRedeemDO::getId, request.getId());
        wrapper.set(RewardRedeemDO::getRedeemType, request.getRedeemType());
        wrapper.set(RewardRedeemDO::getExpenses, request.getExpenses());
        wrapper.set(RewardRedeemDO::getCurrency, request.getCurrency());
        wrapper.set(RewardRedeemDO::getRecipientName, request.getRecipientName());
        wrapper.set(RewardRedeemDO::getRecipientAddress, request.getRecipientAddress());
        wrapper.set(RewardRedeemDO::getRecipientPhoneNum, request.getRecipientPhoneNum());
        wrapper.set(RewardRedeemDO::getRedeemer, request.getRedeemer());
        wrapper.set(RewardRedeemDO::getRedemptionTime, request.getRedemptionTime());
        wrapper.set(RewardRedeemDO::getCourierCompany, request.getCourierCompany());
        wrapper.set(RewardRedeemDO::getExpressNo, request.getExpressNo());
        wrapper.set(RewardRedeemDO::getExpressDate, request.getExpressDate());
        wrapper.set(RewardRedeemDO::getExpressSender, request.getExpressSender());
        wrapper.set(RewardRedeemDO::getAnnex, request.getAnnex());
        wrapper.set(RewardRedeemDO::getRemark, request.getRemark());
        wrapper.set(RewardRedeemDO::getUpdater, SecurityFrameworkUtils.getLoginUserId());
        wrapper.set(RewardRedeemDO::getUpdateTime, new Date());
        rewardRedeemMapper.update(null, wrapper);
        return true;
    }

    private void validateParam(RewardRedeemUpdateVO request) {
        RewardRedeemDO rewardRedeemDO = getById(request.getId());
        if (rewardRedeemDO == null) {
            //todo 提示修改
            throw exception(ErrorCodeConstants.REWARD_ENDTIME_ERROR);
        }
        if (rewardRedeemDO.getStatus() != RewardRedeemStatusEnum.REDEEMING.getValue()) {
            //todo 提示修改
            throw exception(ErrorCodeConstants.REWARD_ENDTIME_ERROR);
        }
    }
}
